@@ -97,7 +97,7 @@ about_button_cb (GtkWidget        *button,
 		gtk_widget_destroy (pm->priv->about);
 
 	pm->priv->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
-		"name", eog_plugin_engine_get_plugin_name (info),
+		(gtk_minor_version > 10) ? "program-name" : "name", eog_plugin_engine_get_plugin_name (info),
 		"copyright", eog_plugin_engine_get_plugin_copyright (info),
 		"authors", eog_plugin_engine_get_plugin_authors (info),
 		"comments", eog_plugin_engine_get_plugin_description (info),
@@ -499,6 +499,9 @@ create_tree_popup_menu (EogPluginManager *pm)
 
 	info = plugin_manager_get_selected_plugin (pm);
 
+	if (info == NULL)
+		return NULL;
+
 	menu = gtk_menu_new ();
 
 	item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
@@ -637,7 +640,10 @@ show_tree_popup_menu (GtkTreeView        *tree,
 		gtk_widget_destroy (pm->priv->popup_menu);
 
 	pm->priv->popup_menu = create_tree_popup_menu (pm);
-	
+
+	if (pm->priv->popup_menu == NULL)
+		return;	
+
 	gtk_menu_attach_to_widget (GTK_MENU (pm->priv->popup_menu),
 				   GTK_WIDGET (pm),
 				   (GtkMenuDetachFunc) tree_popup_menu_detach);
@@ -815,10 +821,8 @@ static void
 eog_plugin_manager_init (EogPluginManager *pm)
 {
 	GtkWidget *label;
-	GtkWidget *alignment;
 	GtkWidget *viewport;
 	GtkWidget *hbuttonbox;
-	gchar *markup;
 
 	eog_debug (DEBUG_PLUGINS);
 
@@ -826,12 +830,7 @@ eog_plugin_manager_init (EogPluginManager *pm)
 
 	gtk_box_set_spacing (GTK_BOX (pm), 6);
 
-	label = gtk_label_new (NULL);
-	markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-					  _("Active plugins"));
-	gtk_label_set_markup (GTK_LABEL (label), markup);
-
-	g_free (markup);
+	label = gtk_label_new_with_mnemonic (_("Active _Plugins:"));
 
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 
@@ -839,10 +838,6 @@ eog_plugin_manager_init (EogPluginManager *pm)
 	
 	gtk_box_pack_start (GTK_BOX (pm), label, FALSE, TRUE, 0);
 	
-	alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
-	gtk_box_pack_start (GTK_BOX (pm), alignment, TRUE, TRUE, 0);
-	
 	viewport = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (viewport),
 					GTK_POLICY_AUTOMATIC,
@@ -850,11 +845,13 @@ eog_plugin_manager_init (EogPluginManager *pm)
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (viewport), 
 					     GTK_SHADOW_IN);
 
-	gtk_container_add (GTK_CONTAINER (alignment), viewport);
+	gtk_box_pack_start (GTK_BOX (pm), viewport, TRUE, TRUE, 0);
 
 	pm->priv->tree = gtk_tree_view_new ();
 	gtk_container_add (GTK_CONTAINER (viewport), pm->priv->tree);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), pm->priv->tree);
+
 	hbuttonbox = gtk_hbutton_box_new ();
 
 	gtk_box_pack_start (GTK_BOX (pm), hbuttonbox, FALSE, FALSE, 0);
